@@ -36,6 +36,7 @@ from trac.util.text import console_print, exception_to_unicode, printout, \
 from trac.util.translation import _, ngettext, has_babel, cleandoc_
 from trac.versioncontrol.api import RepositoryManager
 from trac.wiki.admin import WikiAdmin
+from trac.wiki.formatter import MacroError
 from trac.wiki.macros import WikiMacroBase
 
 
@@ -122,7 +123,8 @@ class TracAdmin(cmd.Cmd):
             printerr(exception_to_unicode(e))
             rv = 2
             if self.env_check():
-                self.env.log.error("Exception in trac-admin command: %s",
+                self.env.log.error("Exception in trac-admin command: %r%s",
+                                   line,
                                    exception_to_unicode(e, traceback=True))
         if not self.interactive:
             return rv
@@ -390,7 +392,7 @@ in order to initialize and prepare the project database.
         printout(_("""
  Please specify the connection string for the database to use.
  By default, a local SQLite database is created in the environment
- directory. It is also possible to use an already existing
+ directory. It is also possible to use an existing MySQL or
  PostgreSQL database (check the Trac documentation for the exact
  connection string syntax).
 """))
@@ -550,8 +552,8 @@ class TracAdminHelpMacro(WikiMacroBase):
                 cmd_mgr = AdminCommandManager(self.env)
                 doc = cmd_mgr.get_command_help(arg)
             if not doc:
-                raise TracError(_('Unknown trac-admin command "%(command)s"',
-                                  command=content))
+                raise MacroError(_('Unknown trac-admin command '
+                                   '"%(command)s"', command=content))
         else:
             doc = TracAdmin.all_docs(self.env)
         buf = StringIO.StringIO()
@@ -559,17 +561,22 @@ class TracAdminHelpMacro(WikiMacroBase):
         return html.PRE(buf.getvalue().decode('utf-8'), class_='wiki')
 
 
-def run(args=None):
-    """Main entry point."""
+def _quote_args(args):
+    def quote(arg):
+        if arg.isalnum():
+            return arg
+        return '"\'"'.join("'%s'" % v for v in arg.split("'"))
+    return [quote(arg) for arg in args]
+
+
+def _run(args):
     if args is None:
         args = sys.argv[1:]
-    if has_babel:
-        translation.activate(get_console_locale())
     warn_setuptools_issue()
     admin = TracAdmin()
     if len(args) > 0:
         if args[0] in ('-h', '--help', 'help'):
-            return admin.onecmd(' '.join(['help'] + args[1:]))
+            return admin.onecmd(' '.join(_quote_args(['help'] + args[1:])))
         elif args[0] in ('-v','--version'):
             printout(os.path.basename(sys.argv[0]), TRAC_VERSION)
         else:
@@ -579,12 +586,10 @@ def run(args=None):
             except UnicodeDecodeError:
                 printerr(_("Non-ascii environment path '%(path)s' not "
                            "supported.", path=to_unicode(env_path)))
-                sys.exit(2)
+                return 2
             admin.env_set(env_path)
             if len(args) > 1:
-                s_args = ' '.join(["'%s'" % c for c in args[2:]])
-                command = args[1] + ' ' + s_args
-                return admin.onecmd(command)
+                return admin.onecmd(' '.join(_quote_args(args[1:])))
             else:
                 while True:
                     try:
@@ -595,6 +600,15 @@ def run(args=None):
         return admin.onecmd("help")
 
 
+def run(args=None):
+    """Main entry point."""
+    translation.activate(get_console_locale())
+    try:
+        return _run(args)
+    finally:
+        translation.deactivate()
+
+
 if __name__ == '__main__':
     pkg_resources.require('Trac==%s' % VERSION)
     sys.exit(run())
