@@ -21,8 +21,10 @@ from __future__ import with_statement
 
 import doctest
 import os
+import shutil
 import sys
 import unittest
+import StringIO
 
 try:
     from babel import Locale
@@ -34,11 +36,15 @@ import trac.db.mysql_backend
 import trac.db.postgres_backend
 import trac.db.sqlite_backend
 from trac.config import Configuration
-from trac.core import ComponentManager
+from trac.core import ComponentManager, ComponentMeta, TracError
 from trac.db.api import DatabaseManager, _parse_db_str
 from trac.env import Environment
+from trac.perm import PermissionCache
 from trac.ticket.default_workflow import load_workflow_config_snippet
 from trac.util import translation
+from trac.util.datefmt import utc
+from trac.web.api import _RequestArgs, Request
+from trac.web.session import DetachedSession
 
 
 def Mock(bases=(), *initargs, **kw):
@@ -112,11 +118,92 @@ class MockPerm(object):
     def __call__(self, realm_or_resource, id=False, version=False):
         return self
 
-    def require(self, action, realm_or_resource=None, id=False, version=False):
+    def require(self, action, realm_or_resource=None, id=False, version=False,
+                message=None):
         pass
     assert_permission = require
 
 
+def MockRequest(env, **kwargs):
+    """Request object for testing. Keyword arguments populate an
+    `environ` dictionary and the callbacks.
+
+    If `authname` is specified in a keyword arguments a `PermissionCache`
+    object is created, otherwise if `authname` is not specified or is
+    `None` a `MockPerm` object is used and the `authname` is set to
+    'anonymous'.
+
+    The following keyword arguments are commonly used:
+    :keyword args: dictionary of request arguments
+    :keyword authname: the name of the authenticated user, or 'anonymous'
+    :keyword method: the HTTP request method
+    :keyword path_info: the request path inside the application
+
+    Additionally `format`, `locale`, `lc_time` and `tz` can be
+    specified as keyword arguments.
+
+    :since: 1.0.11
+    """
+
+    authname = kwargs.get('authname')
+    if authname is None:
+        authname = 'anonymous'
+        perm = MockPerm()
+    else:
+        perm = PermissionCache(env, authname)
+
+    args = _RequestArgs()
+    args.update(kwargs.get('args', {}))
+
+    environ = {
+        'trac.base_url': env.abs_href(),
+        'wsgi.url_scheme': 'http',
+        'HTTP_ACCEPT_LANGUAGE': 'en-US',
+        'PATH_INFO': kwargs.get('path_info', '/'),
+        'REQUEST_METHOD': kwargs.get('method', 'GET'),
+        'REMOTE_ADDR': '127.0.0.1',
+        'REMOTE_USER': authname,
+        'SCRIPT_NAME': '/trac.cgi',
+        'SERVER_NAME': 'localhost',
+        'SERVER_PORT': '80',
+    }
+
+    status_sent = []
+    headers_sent = {}
+    response_sent = StringIO.StringIO()
+
+    def start_response(status, headers, exc_info=None):
+        status_sent.append(status)
+        headers_sent.update(dict(headers))
+        return response_sent.write
+
+    req = Mock(Request, environ, start_response)
+    req.status_sent = status_sent
+    req.headers_sent = headers_sent
+    req.response_sent = response_sent
+
+    from trac.web.chrome import Chrome
+    req.callbacks.update({
+        'arg_list': None,
+        'args': lambda req: args,
+        'authname': lambda req: authname,
+        'chrome': Chrome(env).prepare_request,
+        'form_token': lambda req: kwargs.get('form_token'),
+        'languages': Request._parse_languages,
+        'lc_time': lambda req: kwargs.get('lc_time', locale_en),
+        'locale': lambda req: kwargs.get('locale'),
+        'incookie': Request._parse_cookies,
+        'perm': lambda req: perm,
+        'session': lambda req: DetachedSession(env, authname),
+        'tz': lambda req: kwargs.get('tz', utc),
+        'use_xsendfile': False,
+        'xsendfile_header': None,
+        '_inheaders': Request._parse_headers
+    })
+
+    return req
+
+
 class TestSetup(unittest.TestSuite):
     """
     Test suite decorator that allows a fixture to be setup for a complete
@@ -267,6 +354,8 @@ class EnvironmentStub(Environment):
         ComponentManager.__init__(self)
 
         self.systeminfo = []
+        self._old_registry = None
+        self._old_components = None
 
         import trac
         self.path = path
@@ -322,6 +411,7 @@ class EnvironmentStub(Environment):
 
     def reset_db(self, default_data=None):
         """Remove all data from Trac tables, keeping the tables themselves.
+
         :param default_data: after clean-up, initialize with default data
         :return: True upon success
         """
@@ -391,6 +481,46 @@ class EnvironmentStub(Environment):
             pass
         return False
 
+    def clear_component_registry(self):
+        """Clear the component registry.
+
+        The registry entries are saved entries so they can be restored
+        later using the `restore_component_registry` method.
+
+        :since: 1.0.11
+        """
+        self._old_registry = ComponentMeta._registry
+        self._old_components = ComponentMeta._components
+        ComponentMeta._registry = {}
+
+    def restore_component_registry(self):
+        """Restore the component registry.
+
+        The component registry must have been cleared and saved using
+        the `clear_component_registry` method.
+
+        :since: 1.0.11
+        """
+        if self._old_registry is None:
+            raise TracError("The clear_component_registry method must be "
+                            "called first.")
+        ComponentMeta._registry = self._old_registry
+        ComponentMeta._components = self._old_components
+
+    # tearDown helper
+
+    def reset_db_and_disk(self):
+        """Performs a complete environment reset in a robust way.
+
+        The database is reset, then the connections are shut down, and
+        finally all environment files are removed from the disk.
+        """
+        self.env.reset_db()
+        self.env.shutdown() # really closes the db connections
+        shutil.rmtree(self.env.path)
+        if self._old_registry is not None:
+            self.restore_component_registry()
+
     # overridden
 
     def is_component_enabled(self, cls):
@@ -424,6 +554,7 @@ def suite():
     import trac.admin.tests
     import trac.db.tests
     import trac.mimeview.tests
+    import trac.search.tests
     import trac.timeline.tests
     import trac.ticket.tests
     import trac.util.tests
@@ -442,6 +573,7 @@ def suite():
     suite.addTest(trac.admin.tests.suite())
     suite.addTest(trac.db.tests.suite())
     suite.addTest(trac.mimeview.tests.suite())
+    suite.addTest(trac.search.tests.suite())
     suite.addTest(trac.ticket.tests.suite())
     suite.addTest(trac.timeline.tests.suite())
     suite.addTest(trac.util.tests.suite())
