@@ -17,7 +17,6 @@
 from __future__ import with_statement
 
 import csv
-from datetime import datetime
 import pkg_resources
 import re
 from StringIO import StringIO
@@ -40,7 +39,7 @@ from trac.ticket.notification import TicketNotifyEmail
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import as_bool, as_int, get_reporter_id, lazy
 from trac.util.datefmt import (
-    format_datetime, from_utimestamp, to_utimestamp, utc
+    datetime_now, format_datetime, from_utimestamp, to_utimestamp, utc
 )
 from trac.util.html import to_fragment
 from trac.util.text import (
@@ -150,7 +149,7 @@ class TicketModule(Component):
             return self.export_csv(req, ticket, sep='\t',
                                    mimetype='text/tab-separated-values')
         elif key == 'rss':
-            return self.export_rss(req, ticket)
+            return self._export_rss(req, ticket)
 
     # INavigationContributor methods
 
@@ -297,10 +296,12 @@ class TicketModule(Component):
                     SELECT t.id, tc.time, tc.author, t.type, t.summary,
                            tc.field, tc.oldvalue, tc.newvalue
                     FROM ticket_change tc
-                        INNER JOIN ticket t ON t.id = tc.ticket
-                            AND tc.time>=%s AND tc.time<=%s
-                    ORDER BY tc.time, tc.ticket
-                    """ % (ts_start, ts_stop)):
+                    INNER JOIN ticket t ON
+                        t.id = tc.ticket AND tc.time>=%%s AND tc.time<=%%s
+                    LEFT OUTER JOIN enum p ON
+                        p.type='priority' AND p.name=t.priority
+                    ORDER BY tc.time, COALESCE(p.value,'')='', %s, tc.ticket
+                    """ % db.cast('p.value', 'int'), (ts_start, ts_stop)):
                 if not (oldvalue or newvalue):
                     # ignore empty change corresponding to custom field
                     # created (None -> '') or deleted ('' -> None)
@@ -419,7 +420,6 @@ class TicketModule(Component):
     def _render_batched_timeline_event(self, context, field, event):
         tickets, verb, info, summary, status, resolution, type, \
                 description, comment, cid = event[3]
-        tickets = sorted(tickets)
         if field == 'url':
             return context.href.query(id=','.join(str(t) for t in tickets))
         elif field == 'title':
@@ -548,10 +548,14 @@ class TicketModule(Component):
             elif action == 'diff':
                 return self._render_diff(req, ticket, data, text_fields)
         elif action == 'comment-history':
-            cnum = int(req.args['cnum'])
+            cnum = as_int(req.args.get('cnum'), None)
+            if cnum is None:
+                raise TracError(_("Invalid request arguments."))
             return self._render_comment_history(req, ticket, data, cnum)
         elif action == 'comment-diff':
-            cnum = int(req.args['cnum'])
+            cnum = as_int(req.args.get('cnum'), None)
+            if cnum is None:
+                raise TracError(_("Invalid request arguments."))
             return self._render_comment_diff(req, ticket, data, cnum)
         elif 'preview_comment' in req.args:
             field_changes = {}
@@ -829,8 +833,8 @@ class TicketModule(Component):
         `text_fields` is optionally a list of fields of interest, that are
         considered for jumping to the next change.
         """
-        new_version = int(req.args.get('version', 1))
-        old_version = int(req.args.get('old_version', new_version))
+        new_version = as_int(req.args.get('version'), 1)
+        old_version = as_int(req.args.get('old_version'), new_version)
         if old_version > new_version:
             old_version, new_version = new_version, old_version
 
@@ -995,7 +999,8 @@ class TicketModule(Component):
 
     def _get_comment_history(self, req, ticket, cnum):
         history = []
-        for version, date, author, comment in ticket.get_comment_history(cnum):
+        for version, date, author, comment in \
+                ticket.get_comment_history(cnum) or []:
             history.append({
                 'version': version, 'date': date, 'author': author,
                 'comment': _("''Initial version''") if version == 0 else '',
@@ -1025,8 +1030,8 @@ class TicketModule(Component):
     def _render_comment_diff(self, req, ticket, data, cnum):
         """Show differences between two versions of a ticket comment."""
         req.perm(ticket.resource).require('TICKET_VIEW')
-        new_version = int(req.args.get('version', 1))
-        old_version = int(req.args.get('old_version', new_version))
+        new_version = as_int(req.args.get('version'), 1)
+        old_version = as_int(req.args.get('old_version'), new_version)
         if old_version > new_version:
             old_version, new_version = new_version, old_version
         elif old_version == new_version:
@@ -1146,6 +1151,13 @@ class TicketModule(Component):
         return (content.getvalue(), '%s;charset=utf-8' % mimetype)
 
     def export_rss(self, req, ticket):
+        """:deprecated: since 1.0.6, use `_export_rss` instead. Will be
+                        removed in 1.3.1.
+        """
+        content, content_type = self._export_rss(req, ticket)
+        return ''.join(content), content_type
+
+    def _export_rss(self, req, ticket):
         changes = []
         change_summary = {}
 
@@ -1183,7 +1195,8 @@ class TicketModule(Component):
         data = self._prepare_data(req, ticket, absurls=True)
         data['changes'] = changes
         output = Chrome(self.env).render_template(req, 'ticket.rss', data,
-                                                  'application/rss+xml')
+                                                  'application/rss+xml',
+                                                  iterable=True)
         return output, 'application/rss+xml'
 
     # Ticket validation and changes
@@ -1271,7 +1284,7 @@ class TicketModule(Component):
             valid = False
 
         # Validate summary length
-        if len(ticket['summary']) > self.max_summary_size:
+        if len(ticket['summary'] or '') > self.max_summary_size:
             add_warning(req, _("Ticket summary is too long (must be less "
                                "than %(num)s characters)",
                                num=self.max_summary_size))
@@ -1336,7 +1349,7 @@ class TicketModule(Component):
         # -- Save changes
 
         fragment = ''
-        now = datetime.now(utc)
+        now = datetime_now(utc)
         cnum = ticket.save_changes(get_reporter_id(req, 'author'),
                                    req.args.get('comment'), when=now,
                                    replyto=req.args.get('replyto'))
@@ -1481,7 +1494,7 @@ class TicketModule(Component):
                     if 'TICKET_MODIFY' in req.perm(ticket.resource):
                         field['skip'] = False
                         owner_field = field
-            elif name == 'milestone':
+            elif name == 'milestone' and not field.get('custom'):
                 milestones = [Milestone(self.env, opt)
                               for opt in field['options']]
                 milestones = [m for m in milestones
