@@ -17,16 +17,17 @@
 from __future__ import with_statement
 
 import os.path
-import time
 
 from trac.admin import AdminCommandError, IAdminCommandProvider, get_dir_list
 from trac.config import ConfigSection, ListOption, Option
 from trac.core import *
 from trac.resource import IResourceManager, Resource, ResourceNotFound
 from trac.util.concurrency import threading
+from trac.util.datefmt import time_now
 from trac.util.text import printout, to_unicode, exception_to_unicode
 from trac.util.translation import _
 from trac.web.api import IRequestFilter
+from trac.web.chrome import ITemplateProvider
 
 
 def is_default(reponame):
@@ -230,6 +231,13 @@ class DbRepositoryProvider(Component):
         if is_default(target):
             target = ''
         rm = RepositoryManager(self.env)
+        repositories = rm.get_all_repositories()
+        if target not in repositories:
+            raise TracError(_("Repository \"%(repo)s\" doesn't exist",
+                              repo=target or '(default)'))
+        if 'alias' in repositories[target]:
+            raise TracError(_('Cannot create an alias to the alias "%(repo)s"',
+                              repo=target or '(default)'))
         with self.env.db_transaction as db:
             id = rm.get_repository_id(reponame)
             db.executemany(
@@ -243,6 +251,11 @@ class DbRepositoryProvider(Component):
         if is_default(reponame):
             reponame = ''
         rm = RepositoryManager(self.env)
+        repositories = rm.get_all_repositories()
+        if any(reponame == repos.get('alias')
+               for repos in repositories.itervalues()):
+            raise TracError(_('Cannot remove the repository "%(repos)s" used '
+                              'in aliases', repos=reponame or '(default)'))
         with self.env.db_transaction as db:
             id = rm.get_repository_id(reponame)
             db("DELETE FROM repository WHERE id=%s", (id,))
@@ -258,6 +271,13 @@ class DbRepositoryProvider(Component):
         if is_default(new_reponame):
             new_reponame = ''
         rm = RepositoryManager(self.env)
+        if reponame != new_reponame:
+            repositories = rm.get_all_repositories()
+            if any(reponame == repos.get('alias')
+                   for repos in repositories.itervalues()):
+                raise TracError(_('Cannot rename the repository "%(repos)s" '
+                                  'used in aliases',
+                                  repos=reponame or '(default)'))
         with self.env.db_transaction as db:
             id = rm.get_repository_id(reponame)
             if reponame != new_reponame:
@@ -288,7 +308,8 @@ class DbRepositoryProvider(Component):
 class RepositoryManager(Component):
     """Version control system manager."""
 
-    implements(IRequestFilter, IResourceManager, IRepositoryProvider)
+    implements(IRequestFilter, IResourceManager, IRepositoryProvider,
+               ITemplateProvider)
 
     connectors = ExtensionPoint(IRepositoryConnector)
     providers = ExtensionPoint(IRepositoryProvider)
@@ -349,7 +370,7 @@ class RepositoryManager(Component):
         from trac.web.chrome import Chrome, add_warning
         if handler is not Chrome(self.env):
             for reponame in self.repository_sync_per_request:
-                start = time.time()
+                start = time_now()
                 if is_default(reponame):
                     reponame = ''
                 try:
@@ -383,7 +404,7 @@ class RepositoryManager(Component):
                         reponame or '(default)',
                         exception_to_unicode(e, traceback=True))
                 self.log.info("Synchronized '%s' repository in %0.2f seconds",
-                              reponame or '(default)', time.time() - start)
+                              reponame or '(default)', time_now() - start)
         return handler
 
     def post_process_request(self, req, template, data, content_type):
@@ -500,6 +521,15 @@ class RepositoryManager(Component):
         for reponame, info in reponames.iteritems():
             yield (reponame, info)
 
+    # ITemplateProvider methods
+
+    def get_htdocs_dirs(self):
+        return []
+
+    def get_templates_dirs(self):
+        from pkg_resources import resource_filename
+        return [resource_filename('trac.versioncontrol', 'templates')]
+
     # Public API methods
 
     def get_supported_types(self):
@@ -616,7 +646,8 @@ class RepositoryManager(Component):
         hierarchy and return the name of its associated repository.
         """
         while context:
-            if context.resource.realm in ('source', 'changeset'):
+            if context.resource.realm in ('source', 'changeset') and \
+                    context.resource.parent:
                 return context.resource.parent.id
             context = context.parent
 
@@ -685,7 +716,8 @@ class RepositoryManager(Component):
         if not repositories:
             self.log.warn("Found no repositories matching '%s' base.",
                           base or reponame)
-            return
+            return [_("Repository '%(repo)s' not found",
+                      repo=reponame or _("(default)"))]
 
         errors = []
         for repos in sorted(repositories, key=lambda r: r.reponame):
