@@ -25,7 +25,6 @@ from hashlib import md5, sha1
 import os
 import re
 import sys
-import time
 import urllib2
 import urlparse
 
@@ -36,7 +35,9 @@ from trac.core import *
 from trac.web.api import IAuthenticator, IRequestHandler
 from trac.web.chrome import INavigationContributor
 from trac.util import hex_entropy, md5crypt
+from trac.util.compat import crypt
 from trac.util.concurrency import threading
+from trac.util.datefmt import time_now
 from trac.util.translation import _, tag_
 
 
@@ -165,7 +166,7 @@ class LoginModule(Component):
         with self.env.db_transaction as db:
             # Delete cookies older than 10 days
             db("DELETE FROM auth_cookie WHERE time < %s",
-               (int(time.time()) - 86400 * 10,))
+               (int(time_now()) - 86400 * 10,))
             # Insert a new cookie if we haven't already got one
             cookie = None
             trac_auth = req.incookie.get('trac_auth')
@@ -178,7 +179,7 @@ class LoginModule(Component):
                     INSERT INTO auth_cookie (cookie, name, ipnr, time)
                          VALUES (%s, %s, %s, %s)
                    """, (cookie, remote_user, req.remote_addr,
-                         int(time.time())))
+                         int(time_now())))
         req.authname = remote_user
         req.outcookie['trac_auth'] = cookie
         req.outcookie['trac_auth']['path'] = self.auth_cookie_path \
@@ -302,15 +303,7 @@ class BasicAuthentication(PasswordFileAuthentication):
     def __init__(self, htpasswd, realm):
         # FIXME pass a logger
         self.realm = realm
-        try:
-            import crypt
-            self.crypt = crypt.crypt
-        except ImportError:
-            try:
-                import fcrypt
-                self.crypt = fcrypt.crypt
-            except ImportError:
-                self.crypt = None
+        self.crypt = crypt
         PasswordFileAuthentication.__init__(self, htpasswd)
 
     def load(self, filename):
