@@ -48,18 +48,20 @@ from trac.core import *
 from trac.env import IEnvironmentSetupParticipant, ISystemInfoProvider
 from trac.mimeview.api import RenderingContext, get_mimetype
 from trac.resource import *
-from trac.util import compat, get_reporter_id, html, presentation, \
-                      get_pkginfo, pathjoin, translation
+from trac.util import as_bool, as_int, compat, get_reporter_id, html,\
+                      presentation, get_pkginfo, pathjoin, translation
 from trac.util.html import escape, plaintext
 from trac.util.text import pretty_size, obfuscate_email_address, \
                            shorten_line, unicode_quote_plus, to_unicode, \
                            javascript_quote, exception_to_unicode, to_js_string
 from trac.util.datefmt import (
-    pretty_timedelta, format_datetime, format_date, format_time,
+    pretty_timedelta, datetime_now, format_datetime, format_date, format_time,
     from_utimestamp, http_date, utc, get_date_format_jquery_ui, is_24_hours,
     get_time_format_jquery_ui, user_time, get_month_names_jquery_ui,
     get_day_names_jquery_ui, get_timezone_list_jquery_ui,
-    get_first_week_day_jquery_ui)
+    get_first_week_day_jquery_ui, get_timepicker_separator_jquery_ui,
+    get_period_names_jquery_ui)
+from trac.util.html import to_fragment
 from trac.util.translation import _, get_available_locales
 from trac.web.api import IRequestHandler, ITemplateStreamFilter, HTTPNotFound
 from trac.web.href import Href
@@ -143,7 +145,7 @@ def add_stylesheet(req, filename, mimetype='text/css', media=None):
     application root path. If it is relative, the link will be based off the
     `/chrome/` path.
     """
-    href = _chrome_resource_path(req, filename)
+    href = chrome_resource_path(req, filename)
     add_link(req, 'stylesheet', href, mimetype=mimetype, media=media)
 
 
@@ -161,7 +163,7 @@ def add_script(req, filename, mimetype='text/javascript', charset='utf-8',
     if filename in scriptset:
         return False # Already added that script
 
-    href = _chrome_resource_path(req, filename)
+    href = chrome_resource_path(req, filename)
     script = {'href': href, 'type': mimetype, 'charset': charset,
               'prefix': Markup('<!--[if %s]>' % ie_if) if ie_if else None,
               'suffix': Markup('<![endif]-->') if ie_if else None}
@@ -194,11 +196,7 @@ def add_warning(req, msg, *args):
     the message is escaped (and therefore converted to `Markup`) before it is
     stored in the request object.
     """
-    if args:
-        msg %= args
-    msg = escape(msg, False)
-    if msg not in req.chrome['warnings']:
-        req.chrome['warnings'].append(msg)
+    _add_message(req, 'warnings', msg, args)
 
 
 def add_notice(req, msg, *args):
@@ -208,11 +206,16 @@ def add_notice(req, msg, *args):
     the message is escaped (and therefore converted to `Markup`) before it is
     stored in the request object.
     """
+    _add_message(req, 'notices', msg, args)
+
+
+def _add_message(req, name, msg, args):
     if args:
         msg %= args
-    msg = escape(msg, False)
-    if msg not in req.chrome['notices']:
-        req.chrome['notices'].append(msg)
+    if not isinstance(msg, Markup):
+        msg = Markup(to_fragment(msg))
+    if msg not in req.chrome[name]:
+        req.chrome[name].append(msg)
 
 
 def add_ctxtnav(req, elm_or_label, href=None, title=None):
@@ -351,7 +354,7 @@ def chrome_info_script(req, use_late=None):
     return fragment
 
 
-def _chrome_resource_path(req, filename):
+def chrome_resource_path(req, filename):
     """Get the path for a chrome resource given its `filename`.
 
     If `filename` is a network-path reference (i.e. starts with a protocol
@@ -369,6 +372,9 @@ def _chrome_resource_path(req, filename):
         return href(filename)
 
 
+_chrome_resource_path = chrome_resource_path  # will be removed in 1.3.1
+
+
 def _save_messages(req, url, permanent):
     """Save warnings and notices in case of redirect, so that they can
     be displayed after the redirect."""
@@ -555,6 +561,11 @@ class Chrome(Component):
         format. (''since 1.0'')
         """)
 
+    use_chunked_encoding = BoolOption('trac', 'use_chunked_encoding', 'false',
+        """If enabled, send contents as chunked encoding in HTTP/1.1.
+        Otherwise, send contents with `Content-Length` header after entire of
+        the contents are rendered. (''since 1.0.6'')""")
+
     templates = None
 
     # DocType for 'text/html' output
@@ -565,6 +576,8 @@ class Chrome(Component):
         '_': translation.gettext,
         'all': all,
         'any': any,
+        'as_bool': as_bool,
+        'as_int': as_int,
         'classes': presentation.classes,
         'date': datetime.date,
         'datetime': datetime.datetime,
@@ -690,11 +703,11 @@ class Chrome(Component):
     def get_htdocs_dirs(self):
         return [('common', pkg_resources.resource_filename('trac', 'htdocs')),
                 ('shared', self.shared_htdocs_dir),
-                ('site', self.env.get_htdocs_dir())]
+                ('site', self.env.htdocs_dir)]
 
     def get_templates_dirs(self):
         return filter(None, [
-            self.env.get_templates_dir(),
+            self.env.templates_dir,
             self.shared_templates_dir,
             pkg_resources.resource_filename('trac', 'templates'),
         ])
@@ -1023,7 +1036,7 @@ class Chrome(Component):
         return self.templates.load(filename, cls=cls)
 
     def render_template(self, req, filename, data, content_type=None,
-                        fragment=False):
+                        fragment=False, iterable=False):
         """Render the `filename` using the `data` for the context.
 
         The `content_type` argument is used to choose the kind of template
@@ -1034,6 +1047,9 @@ class Chrome(Component):
 
         When `fragment` is specified, the (filtered) Genshi stream is
         returned.
+
+        When `iterable` is specified, the content as an iterable instance
+        which is generated from filtered Genshi stream is returned.
         """
         if content_type is None:
             content_type = 'text/html'
@@ -1090,6 +1106,9 @@ class Chrome(Component):
             'late_script_data': req.chrome['script_data'],
         })
 
+        if iterable:
+            return self.iterable_content(stream, method, doctype=doctype)
+
         try:
             buffer = StringIO()
             stream.render(method, doctype=doctype, out=buffer,
@@ -1127,12 +1146,12 @@ class Chrome(Component):
 
         files = {}
         # Collect templates list
-        site_templates = list_dir(self.env.get_templates_dir(), '.html')
+        site_templates = list_dir(self.env.templates_dir, '.html')
         shared_templates = list_dir(Chrome(self.env).shared_templates_dir,
                                     '.html')
 
         # Collect static resources list
-        site_htdocs = list_dir(self.env.get_htdocs_dir())
+        site_htdocs = list_dir(self.env.htdocs_dir)
         shared_htdocs = list_dir(Chrome(self.env).shared_htdocs_dir)
 
         if any((site_templates, shared_templates, site_htdocs, shared_htdocs)):
@@ -1144,12 +1163,39 @@ class Chrome(Component):
             }
         return files
 
+    def iterable_content(self, stream, method, **kwargs):
+        """Generate an iterable object which iterates `str` instances
+        from the given stream instance.
+
+        :param method: the serialization method; can be either "xml",
+                       "xhtml", "html", "text", or a custom serializer
+                       class
+        """
+        try:
+            if method == 'text':
+                for chunk in stream.serialize(method, **kwargs):
+                    yield chunk.encode('utf-8')
+            else:
+                for chunk in stream.serialize(method, **kwargs):
+                    yield chunk.encode('utf-8') \
+                               .translate(_translate_nop,
+                                          _invalid_control_chars)
+        except Exception, e:
+            pos = self._stream_location(stream)
+            if pos:
+                location = "'%s', line %s, char %s" % pos
+            else:
+                location = '(unknown template location)'
+            self.log.error('Genshi %s error while rendering template %s%s',
+                           e.__class__.__name__, location,
+                           exception_to_unicode(e, traceback=True))
+
     # E-mail formatting utilities
 
     def cc_list(self, cc_field):
         """Split a CC: value in a list of addresses."""
         ccs = []
-        for cc in re.split(r'[;,]', cc_field):
+        for cc in re.split(r'[;,]', cc_field or ''):
             cc = cc.strip()
             if cc:
                 ccs.append(cc)
@@ -1228,19 +1274,28 @@ class Chrome(Component):
         add_script(req, 'common/js/jquery-ui-addons.js')
         add_stylesheet(req, 'common/css/jquery-ui-addons.css')
         is_iso8601 = req.lc_time == 'iso8601'
+        now = datetime_now(req.tz)
+        tzoffset = now.strftime('%z')
+        default_timezone = 'Z' if tzoffset == '+0000' else \
+                           tzoffset[:-2] + ':' + tzoffset[-2:]
+        if is_iso8601:
+            timezone_list = get_timezone_list_jquery_ui(now)
+        else:
+            # default timezone must be included
+            timezone_list = [default_timezone if default_timezone != 'Z' else
+                             {'value': 'Z', 'label': '+00:00'}]
         add_script_data(req, jquery_ui={
             'month_names': get_month_names_jquery_ui(req),
             'day_names': get_day_names_jquery_ui(req),
             'date_format': get_date_format_jquery_ui(req.lc_time),
             'time_format': get_time_format_jquery_ui(req.lc_time),
             'ampm': not is_24_hours(req.lc_time),
+            'period_names': get_period_names_jquery_ui(req),
             'first_week_day': get_first_week_day_jquery_ui(req),
-            'timepicker_separator': 'T' if is_iso8601 else ' ',
+            'timepicker_separator': get_timepicker_separator_jquery_ui(req),
             'show_timezone': is_iso8601,
-            # default timezone must be included
-            'timezone_list': get_timezone_list_jquery_ui() \
-                             if is_iso8601 \
-                             else [{'value': 'Z', 'label': '+00:00'}],
+            'default_timezone': default_timezone,
+            'timezone_list': timezone_list,
             'timezone_iso8601': is_iso8601,
         })
         add_script(req, 'common/js/jquery-ui-i18n.js')
