@@ -27,7 +27,9 @@ import pkg_resources
 from pprint import pformat, pprint
 import re
 import sys
+import traceback
 import StringIO
+from urlparse import urlparse
 
 from genshi.builder import tag
 from genshi.output import DocType
@@ -52,7 +54,7 @@ from trac.util.translation import _, get_negotiated_locale, has_babel, \
 from trac.web.api import HTTPBadRequest, HTTPException, HTTPForbidden, \
                          HTTPInternalError, HTTPNotFound, IAuthenticator, \
                          IRequestFilter, IRequestHandler, Request, \
-                         RequestDone
+                         RequestDone, TracNotImplementedError
 from trac.web.chrome import Chrome, add_notice, add_warning
 from trac.web.href import Href
 from trac.web.session import Session
@@ -184,12 +186,12 @@ class RequestDispatcher(Component):
         req.callbacks.update({
             'authname': self.authenticate,
             'chrome': chrome.prepare_request,
+            'form_token': self._get_form_token,
+            'lc_time': self._get_lc_time,
+            'locale': self._get_locale,
             'perm': self._get_perm,
             'session': self._get_session,
-            'locale': self._get_locale,
-            'lc_time': self._get_lc_time,
             'tz': self._get_timezone,
-            'form_token': self._get_form_token,
             'use_xsendfile': self._get_use_xsendfile,
             'xsendfile_header': self._get_xsendfile_header,
         })
@@ -286,6 +288,12 @@ class RequestDispatcher(Component):
             raise HTTPForbidden(e)
         except ResourceNotFound, e:
             raise HTTPNotFound(e)
+        except NotImplementedError, e:
+            tb = traceback.extract_tb(err[2])[-1]
+            self.log.warning("%s caught from %s:%d in %s: %s",
+                             e.__class__.__name__, tb[0], tb[1], tb[2],
+                             to_unicode(e) or "(no message)")
+            raise HTTPInternalError(TracNotImplementedError(e))
         except TracError, e:
             raise HTTPInternalError(e)
 
@@ -612,9 +620,10 @@ def send_internal_error(env, req, exc_info):
             faulty_plugins.sort(key=lambda p: p['frame_idx'])
             if faulty_plugins:
                 info = faulty_plugins[0]['info']
+                home_page = info.get('home_page', '')
                 if 'trac' in info:
                     tracker = info['trac']
-                elif info.get('home_page', '').startswith(th):
+                elif urlparse(home_page)[1] == urlparse(th)[1]:
                     tracker = th
                     plugin_name = info.get('home_page', '').rstrip('/') \
                                                            .split('/')[-1]
