@@ -110,6 +110,10 @@ def _markup_to_unicode(markup):
     return to_unicode(markup)
 
 
+class MacroError(TracError):
+    pass
+
+
 class ProcessorError(TracError):
     pass
 
@@ -616,7 +620,7 @@ class Formatter(object):
     def _make_lhref_link(self, match, fullmatch, rel, ns, target, label):
         if not label: # e.g. `[http://target]` or `[wiki:target]`
             if target:
-                if target.startswith('//'):     # for `[http://target]`
+                if ns and target.startswith('//'):     # for `[http://target]`
                     label = ns + ':' + target   #  use `http://target`
                 else:                           # for `wiki:target`
                     label = target.lstrip('/')  #  use only `target`
@@ -724,7 +728,8 @@ class Formatter(object):
         interwiki = InterWikiMap(self.env)
         if ns in interwiki:
             url, title = interwiki.url(ns, target)
-            return self._make_ext_link(url, label, title)
+            if url:
+                return self._make_ext_link(url, label, title)
 
     def _make_ext_link(self, url, text, title=''):
         local_url = self.env.project_url or \
@@ -786,6 +791,8 @@ class Formatter(object):
             args = fullmatch.group('macroargs')
         try:
             return macro.ensure_inline(macro.process(args))
+        except MacroError, e:
+            return system_message(e)
         except Exception, e:
             self.env.log.error('Macro %s(%s) failed:%s', name, args,
                                exception_to_unicode(e, traceback=True))
