@@ -23,11 +23,11 @@ from functools import partial
 import re
 from subprocess import Popen, PIPE
 from threading import Lock
-import time
 import weakref
 
 from trac.util import terminate
 from trac.util.compat import close_fds
+from trac.util.datefmt import time_now
 from trac.util.text import to_unicode
 
 __all__ = ['GitError', 'GitErrorSha', 'Storage', 'StorageFactory']
@@ -86,6 +86,13 @@ def _unquote(path):
     return path
 
 
+def _close_proc_pipes(proc):
+    if proc:
+        for f in (proc.stdin, proc.stdout, proc.stderr):
+            if f:
+                f.close()
+
+
 class GitCore(object):
     """Low-level wrapper around git executable"""
 
@@ -126,6 +133,9 @@ class GitCore(object):
         return cmd
 
     def __pipe(self, git_cmd, *cmd_args, **kw):
+        kw.setdefault('stdin', PIPE)
+        kw.setdefault('stdout', PIPE)
+        kw.setdefault('stderr', PIPE)
         return Popen(self.__build_git_cmd(git_cmd, *cmd_args),
                      close_fds=close_fds, **kw)
 
@@ -136,6 +146,7 @@ class GitCore(object):
 
         p = self.__pipe(git_cmd, stdout=PIPE, stderr=PIPE, *cmd_args)
         stdout_data, stderr_data = p.communicate()
+        _close_proc_pipes(p)
         if self.__log and (p.returncode != 0 or stderr_data):
             self.__log.debug('%s exits with %d, dir: %r, args: %s %r, '
                              'stderr: %r', self.__git_bin, p.returncode,
@@ -377,7 +388,8 @@ class Storage(object):
             codecs.lookup(git_fs_encoding)
 
             # setup conversion functions
-            self._fs_to_unicode = lambda s: s.decode(git_fs_encoding)
+            self._fs_to_unicode = lambda s: s.decode(git_fs_encoding,
+                                                     'replace')
             self._fs_from_unicode = lambda s: s.encode(git_fs_encoding)
         else:
             # pass bytestrings as-is w/o any conversion
@@ -413,12 +425,15 @@ class Storage(object):
         self.logger.debug("PyGIT.Storage instance for '%s' is constructed",
                           git_dir)
 
+    def _cleanup_proc(self, proc):
+        if proc:
+            _close_proc_pipes(proc)
+            terminate(proc)
+            proc.wait()
+
     def __del__(self):
         with self.__cat_file_pipe_lock:
-            if self.__cat_file_pipe is not None:
-                self.__cat_file_pipe.stdin.close()
-                terminate(self.__cat_file_pipe)
-                self.__cat_file_pipe.wait()
+            self._cleanup_proc(self.__cat_file_pipe)
 
     #
     # cache handling
@@ -460,7 +475,7 @@ class Storage(object):
     def _build_rev_cache(self, refs):
         self.logger.debug("triggered rebuild of commit tree db for '%s'",
                           self.repo_path)
-        ts0 = time.time()
+        ts0 = time_now()
 
         new_db = {} # db
         new_sdb = {} # short_rev db
@@ -547,7 +562,7 @@ class Storage(object):
         rev_cache = self.RevCache(youngest, oldest, new_db, refs, new_sdb)
         self.logger.debug("rebuilt commit tree db for '%s' with %d entries "
                           "(took %.1f ms)", self.repo_path, len(new_db),
-                          1000 * (time.time() - ts0))
+                          1000 * (time_now() - ts0))
         return rev_cache
 
     def _get_refs(self):
@@ -679,9 +694,7 @@ class Storage(object):
                 # consistent state (Otherwise it happens that next time we
                 # call cat_file we get payload from previous call)
                 self.logger.debug("closing cat_file pipe")
-                self.__cat_file_pipe.stdin.close()
-                terminate(self.__cat_file_pipe)
-                self.__cat_file_pipe.wait()
+                self._cleanup_proc(self.__cat_file_pipe)
                 self.__cat_file_pipe = None
 
     def verifyrev(self, rev):
@@ -919,9 +932,8 @@ class Storage(object):
                             path, _ = path.rsplit('/', 1)
                         except ValueError:
                             break
-            f.close()
-            terminate(p[0])
-            p[0].wait()
+            if p:
+                self._cleanup_proc(p[0])
             p[:] = []
             while True:
                 yield None
@@ -939,9 +951,7 @@ class Storage(object):
             yield historian
         finally:
             if p:
-                p[0].stdout.close()
-                terminate(p[0])
-                p[0].wait()
+                self._cleanup_proc(p[0])
 
     def last_change(self, sha, path, historian=None):
         if historian is not None:
